==> mosaicml_cli-0.2.12-py3-none-any.whl: mcli/config.py <==
"""Global Singleton Config Store"""
from __future__ import annotations

import logging
import os
from dataclasses import asdict, dataclass, field
from datetime import datetime, timedelta
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List

import ruamel.yaml
import yaml
from ruamel.yaml import YAML
from ruamel.yaml.comments import CommentedMap

from mcli.api.exceptions import MCLIConfigError
from mcli.models import MCLICluster, MCLIEnvVar
from mcli.utils.utils_logging import FAIL
from mcli.utils.utils_modules import check_if_module_exists
from mcli.utils.utils_serializable_dataclass import SerializableDataclass
from mcli.utils.utils_yaml import StringDumpYAML


def env_path_override_config(config_value: str):
    if config_value in os.environ:
        globals()[config_value] = Path(os.environ[config_value])


def env_str_override_config(config_value: str):
    if config_value in os.environ:
        globals()[config_value] = os.environ[config_value]


MCLI_CONFIG_DIR: Path = Path(os.path.expanduser('~/.mosaic'))
env_path_override_config('MCLI_CONFIG_DIR')

MCLI_BACKUP_CONFIG_DIR: Path = Path(os.path.expanduser('~/.mosaic.bak'))
env_path_override_config('MCLI_BACKUP_CONFIG_DIR')

MCTL_CONFIG_PATH: Path = MCLI_CONFIG_DIR / 'config'
env_path_override_config('MCTL_CONFIG_PATH')

MOSAICML_API_ENDPOINT: str = 'http://localhost:3001/graphql'
env_str_override_config('MOSAICML_API_ENDPOINT')

MCLI_CONFIG_PATH: Path = MCLI_CONFIG_DIR / 'mcli_config'
env_path_override_config('MCLI_CONFIG_PATH')

MCLI_KUBECONFIG: Path = MCLI_CONFIG_DIR / 'kube_config'
env_path_override_config('MCLI_KUBECONFIG')

COMPOSER_INSTALLED: bool = check_if_module_exists('composer')

UPDATE_CHECK_FREQUENCY_DAYS: float = 2

PAGER_LIMIT: int = 50  # When `mcli get` returns more than PAGER_LIMIT entries, a pager should be used

JOB_TTL: int = int(timedelta(days=14).total_seconds())
MCLI_MODE_ENV: str = 'MCLI_MODE'
env_path_override_config('MCLI_MODE_ENV')

# Used for local dev and testing
MOSAICML_API_KEY_ENV: str = 'MOSAICML_API_KEY'

SUPPORT_TOKEN_KEY_ENV: str = 'MOSAICML_SUPPORT_TOKEN_KEY'

logging.getLogger('urllib3.connectionpool').disabled = True

logger = logging.getLogger(__name__)


class FeatureFlag(Enum):
    ALPHA_TESTER = 'ALPHA_TESTER'
    USE_FEATUREDB = 'USE_FEATUREDB'
    MLPERF_MODE = 'MLPERF_MODE'
    USE_DEMO_NODES = 'USE_DEMO_NODES'


class MCLIMode(Enum):
    PROD = 'PROD'
    DEV = 'DEV'
    INTERNAL = 'INTERNAL'


@dataclass
class MCLIConfig(SerializableDataclass):
    """Global Config Store persisted on local disk"""

    # set to default for now to not break existing users' configs
    MOSAICML_API_KEY: str = ''  # pylint: disable=invalid-name Global Stored within Singleton

    feature_flags: Dict[str, bool] = field(default_factory=dict)
    last_update_check: datetime = field(default_factory=datetime.now)

    # Global Environment Variables
    environment_variables: List[MCLIEnvVar] = field(default_factory=list)

    # Registered Clusters
    clusters: List[MCLICluster] = field(default_factory=list)

    @classmethod
    def empty(cls) -> MCLIConfig:
        conf = MCLIConfig()
        conf.apply_environment_overrides()
        return conf

    @property
    def dev_mode(self) -> bool:
        return self.mcli_mode == MCLIMode.DEV

    @property
    def internal(self) -> bool:
        return self.mcli_mode in (
            MCLIMode.INTERNAL,
            MCLIMode.DEV,
        )

    @property
    def mcli_mode(self) -> MCLIMode:
        if os.environ.get(MCLI_MODE_ENV, None):
            found_mode = os.environ.get(MCLI_MODE_ENV)
            if found_mode == 'DEV':
                return MCLIMode.DEV
            elif found_mode == 'INTERNAL':
                return MCLIMode.INTERNAL

        if os.environ.get("DOGEMODE", None) == 'ON':
            return MCLIMode.INTERNAL

        return MCLIMode.PROD

    def apply_environment_overrides(self):
        api_key_env = os.environ.get(MOSAICML_API_KEY_ENV, None)
        if api_key_env is not None:
            self.MOSAICML_API_KEY = api_key_env

    @staticmethod
    def _remove_deprecated_cluster_details(clusters: List[Dict[str, str]]) -> List[Dict[str, str]]:
        # pylint: disable-next=import-outside-toplevel
        from mcli.serverside.clusters.cluster import GenericK8sCluster

        valid_clusters: List[Dict[str, str]] = []
        for cluster in clusters:
            if 'kubernetes_context' in cluster and cluster[
                    'kubernetes_context'] not in GenericK8sCluster.get_k8s_context_map():
                continue
            if 'environment_overrides' in cluster:
                del cluster['environment_overrides']
            valid_clusters.append(cluster)
        return valid_clusters

    @classmethod
    def load_config(cls, safe: bool = False) -> MCLIConfig:
        """Loads the MCLIConfig from local disk


        Args:
            safe (bool): If safe is true, if the config fails to load it will return
                an empty generated config

        Return:
            Returns the MCLIConfig if successful, otherwise raises Exception
        """
        try:
            with open(MCLI_CONFIG_PATH, 'r', encoding='utf8') as f:
                data: Dict[str, Any] = yaml.full_load(f)
                # TODO: Remove after full deprecation transition
                if 'dev_mode' in data:
                    del data['dev_mode']
                if 'internal' in data:
                    del data['internal']
                # TODO(END): Remove after full deprecation transition
                if 'platforms' in data:
                    data['clusters'] = data['platforms']
                    del data['platforms']
                data['clusters'] = cls._remove_deprecated_cluster_details(data.get('clusters', []))
            conf: MCLIConfig = MCLIConfig.from_dict(data)
        except FileNotFoundError as e:
            if safe:
                return MCLIConfig.empty()
            raise MCLIConfigError(
                f'No mcli config file found at: {MCLI_CONFIG_PATH}. Please run `mcli init` to create it.') from e

        # Optional values can get filled in over time. If a new optional value is not
        # present in the config, let it be filled in by the default, if one was set.
        if set(asdict(conf)) != set(data):
            # TODO: Bug on over-saving HEK-452
            conf.save_config()

        conf.apply_environment_overrides()

        return conf

    def save_config(self) -> bool:
        """Saves the MCLIConfig to local disk

        Return:
            Returns true if successful
        """
        if self.dev_mode:
            print('saving config...')

        data = self._get_formatted_dump()
        y = YAML()
        y.explicit_start = True  # type: ignore
        with open(MCLI_CONFIG_PATH, 'w', encoding='utf8') as f:
            y.dump(data, f)
        return True

    def _get_formatted_dump(self) -> CommentedMap:
        """Gets the ruamel yaml formatted dump of the config
        """
        raw_data = self.to_disk()

        # Moves clusters to last item
        clusters = raw_data['clusters']
        del raw_data['clusters']
        raw_data['clusters'] = clusters

        # Remove invalid clusters

        data: CommentedMap = ruamel.yaml.load(
            yaml.dump(raw_data),
            ruamel.yaml.RoundTripLoader,
        )
        data.yaml_set_start_comment('MCLI Config Data\n')
        data.yaml_set_comment_before_after_key(
            key='environment_variables',
            before='\nAll Global Environment variables go here',
        )
        data.yaml_set_comment_before_after_key(
            key='clusters',
            before='\nAll Clusters configured for MCLI',
        )
        return data

    def feature_enabled(self, feature: FeatureFlag) -> bool:
        """Checks if the feature flag is enabled

        Args:
            feature (FeatureFlag): The feature to check
        """
        if not self.internal:
            return False

        if feature.value in self.feature_flags:
            enabled = self.feature_flags[feature.value]
            return bool(enabled)
        return False

    def __str__(self) -> str:
        data = self._get_formatted_dump()
        y = StringDumpYAML()
        return y.dump(data)


def feature_enabled(feature: FeatureFlag) -> bool:
    conf = MCLIConfig.load_config(safe=True)
    return conf.feature_enabled(feature=feature)


class Messages():
    MCLI_NOT_INITIALIZED = f'{FAIL} MCLI not yet initialized. Please run `mcli init` first.'


MESSAGE = Messages()

==> mosaicml_cli-0.2.14-py3-none-any.whl: mcli/config.py <==
"""Global Singleton Config Store"""
from __future__ import annotations

import logging
import os
from dataclasses import asdict, dataclass, field
from datetime import datetime, timedelta
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List

import ruamel.yaml
import yaml
from ruamel.yaml import YAML
from ruamel.yaml.comments import CommentedMap

from mcli.api.exceptions import MCLIConfigError
from mcli.models import MCLICluster, MCLIEnvVar
from mcli.utils.utils_logging import FAIL
from mcli.utils.utils_modules import check_if_module_exists
from mcli.utils.utils_serializable_dataclass import SerializableDataclass
from mcli.utils.utils_yaml import StringDumpYAML


def env_path_override_config(config_value: str):
    if config_value in os.environ:
        globals()[config_value] = Path(os.environ[config_value])


def env_str_override_config(config_value: str):
    if config_value in os.environ:
        globals()[config_value] = os.environ[config_value]


MCLI_CONFIG_DIR: Path = Path(os.path.expanduser('~/.mosaic'))
env_path_override_config('MCLI_CONFIG_DIR')

MCLI_BACKUP_CONFIG_DIR: Path = Path(os.path.expanduser('~/.mosaic.bak'))
env_path_override_config('MCLI_BACKUP_CONFIG_DIR')

MCTL_CONFIG_PATH: Path = MCLI_CONFIG_DIR / 'config'
env_path_override_config('MCTL_CONFIG_PATH')

MOSAICML_API_ENDPOINT: str = 'https://api.mosaicml.com/graphql'
MOSAICML_API_ENDPOINT_LOCAL: str = 'http://localhost:3001/graphql'
MOSAICML_API_ENDPOINT_ENV: str = 'MOSAICML_API_ENDPOINT'
env_str_override_config(MOSAICML_API_ENDPOINT_ENV)

MCLI_CONFIG_PATH: Path = MCLI_CONFIG_DIR / 'mcli_config'
env_path_override_config('MCLI_CONFIG_PATH')

MCLI_KUBECONFIG: Path = MCLI_CONFIG_DIR / 'kube_config'
env_path_override_config('MCLI_KUBECONFIG')

COMPOSER_INSTALLED: bool = check_if_module_exists('composer')

UPDATE_CHECK_FREQUENCY_DAYS: float = 2

PAGER_LIMIT: int = 50  # When `mcli get` returns more than PAGER_LIMIT entries, a pager should be used

JOB_TTL: int = int(timedelta(days=14).total_seconds())
MCLI_MODE_ENV: str = 'MCLI_MODE'
env_str_override_config('MCLI_MODE_ENV')

# Used for local dev and testing
MOSAICML_API_KEY_ENV: str = 'MOSAICML_API_KEY'

SUPPORT_TOKEN_KEY_ENV: str = 'MOSAICML_SUPPORT_TOKEN_KEY'

logging.getLogger('urllib3.connectionpool').disabled = True

logger = logging.getLogger(__name__)


class FeatureFlag(Enum):
    ALPHA_TESTER = 'ALPHA_TESTER'
    USE_FEATUREDB = 'USE_FEATUREDB'
    MLPERF_MODE = 'MLPERF_MODE'
    USE_DEMO_NODES = 'USE_DEMO_NODES'
    USE_MCLOUD = 'USE_MCLOUD'


class MCLIMode(Enum):
    PROD = 'PROD'
    DEV = 'DEV'
    INTERNAL = 'INTERNAL'


@dataclass
class MCLIConfig(SerializableDataclass):
    """Global Config Store persisted on local disk"""

    # set to default for now to not break existing users' configs
    MOSAICML_API_KEY: str = ''  # pylint: disable=invalid-name Global Stored within Singleton

    feature_flags: Dict[str, bool] = field(default_factory=dict)
    last_update_check: datetime = field(default_factory=datetime.now)

    # Global Environment Variables
    environment_variables: List[MCLIEnvVar] = field(default_factory=list)

    # Registered Clusters
    clusters: List[MCLICluster] = field(default_factory=list)

    @classmethod
    def empty(cls) -> MCLIConfig:
        conf = MCLIConfig()
        return conf

    @property
    def dev_mode(self) -> bool:
        return self.mcli_mode == MCLIMode.DEV

    @property
    def internal(self) -> bool:
        return self.mcli_mode in (
            MCLIMode.INTERNAL,
            MCLIMode.DEV,
        )

    @property
    def mcli_mode(self) -> MCLIMode:
        if os.environ.get(MCLI_MODE_ENV, None):
            found_mode = os.environ.get(MCLI_MODE_ENV)
            if found_mode == 'DEV':
                return MCLIMode.DEV
            elif found_mode == 'INTERNAL':
                return MCLIMode.INTERNAL

        if os.environ.get('DOGEMODE', None) == 'ON':
            return MCLIMode.INTERNAL

        return MCLIMode.PROD

    @property
    def api_key(self):
        api_key_env = os.environ.get(MOSAICML_API_KEY_ENV, None)
        if api_key_env is not None:
            return api_key_env
        elif self.MOSAICML_API_KEY:
            return self.MOSAICML_API_KEY
        return ''

    @api_key.setter
    def api_key(self, value: str):
        self.MOSAICML_API_KEY = value

    @staticmethod
    def _remove_deprecated_cluster_details(clusters: List[Dict[str, str]]) -> List[Dict[str, str]]:
        # pylint: disable-next=import-outside-toplevel
        from mcli.serverside.clusters.cluster import GenericK8sCluster

        valid_clusters: List[Dict[str, str]] = []
        for cluster in clusters:
            if cluster.get('kubernetes_context', '').endswith('research-01'):
                continue
            if 'kubernetes_context' in cluster and cluster[
                    'kubernetes_context'] not in GenericK8sCluster.get_k8s_context_map():
                continue
            if 'environment_overrides' in cluster:
                del cluster['environment_overrides']
            valid_clusters.append(cluster)
        return valid_clusters

    @classmethod
    def load_config(cls, safe: bool = False) -> MCLIConfig:
        """Loads the MCLIConfig from local disk


        Args:
            safe (bool): If safe is true, if the config fails to load it will return
                an empty generated config

        Return:
            Returns the MCLIConfig if successful, otherwise raises Exception
        """
        try:
            with open(MCLI_CONFIG_PATH, 'r', encoding='utf8') as f:
                data: Dict[str, Any] = yaml.full_load(f)
                # TODO: Remove after full deprecation transition
                if 'dev_mode' in data:
                    del data['dev_mode']
                if 'internal' in data:
                    del data['internal']
                # TODO(END): Remove after full deprecation transition
                if 'platforms' in data:
                    data['clusters'] = data['platforms']
                    del data['platforms']
                data['clusters'] = cls._remove_deprecated_cluster_details(data.get('clusters', []))
            conf: MCLIConfig = MCLIConfig.from_dict(data)
        except FileNotFoundError as e:
            if safe:
                return MCLIConfig.empty()
            raise MCLIConfigError(
                f'No mcli config file found at: {MCLI_CONFIG_PATH}. Please run `mcli init` to create it.') from e

        # Optional values can get filled in over time. If a new optional value is not
        # present in the config, let it be filled in by the default, if one was set.
        if set(asdict(conf)) != set(data):
            # TODO: Bug on over-saving HEK-452
            conf.save_config()

        return conf

    def save_config(self) -> bool:
        """Saves the MCLIConfig to local disk

        Return:
            Returns true if successful
        """
        if self.dev_mode:
            print('saving config...')

        data = self._get_formatted_dump()
        y = YAML()
        y.explicit_start = True  # type: ignore
        with open(MCLI_CONFIG_PATH, 'w', encoding='utf8') as f:
            y.dump(data, f)
        return True

    def _get_formatted_dump(self) -> CommentedMap:
        """Gets the ruamel yaml formatted dump of the config
        """
        raw_data = self.to_disk()

        # Moves clusters to last item
        clusters = raw_data['clusters']
        del raw_data['clusters']
        raw_data['clusters'] = clusters

        # Remove invalid clusters

        data: CommentedMap = ruamel.yaml.load(
            yaml.dump(raw_data),
            ruamel.yaml.RoundTripLoader,
        )
        data.yaml_set_start_comment('MCLI Config Data\n')
        data.yaml_set_comment_before_after_key(
            key='environment_variables',
            before='\nAll Global Environment variables go here',
        )
        data.yaml_set_comment_before_after_key(
            key='clusters',
            before='\nAll Clusters configured for MCLI',
        )
        return data

    def feature_enabled(self, feature: FeatureFlag) -> bool:
        """Checks if the feature flag is enabled

        Args:
            feature (FeatureFlag): The feature to check
        """
        if not self.internal:
            return False

        if feature.value in self.feature_flags:
            enabled = self.feature_flags[feature.value]
            return bool(enabled)
        return False

    def __str__(self) -> str:
        data = self._get_formatted_dump()
        y = StringDumpYAML()
        return y.dump(data)


def feature_enabled(feature: FeatureFlag) -> bool:
    conf = MCLIConfig.load_config(safe=True)
    return conf.feature_enabled(feature=feature)


class Messages():
    MCLI_NOT_INITIALIZED = f'{FAIL} MCLI not yet initialized. Please run `mcli init` first.'
    API_KEY_MISSING = f'{FAIL} API key is missing please create one.'


MESSAGE = Messages()
